from .pythonProject import Lagrange
from .pythonProject import brent
from .pythonProject import golden_ratio
from .pythonProject import parse_variables_tz2
from .pythonProject import parabolic_interpolation
from .pythonProject import plot
from .pythonProject import gradient_descent_fixed
from .pythonProject import local_extrema
from .pythonProject import gradient_descent_fixed
from .pythonProject import parse__restr_function
from .pythonProject import parse_restriction
from .pythonProject import parse_variables
from .pythonProject import parser_task1
from .pythonProject import tz3
from .pythonProject import tz4
from .pythonProject import plot
from .pythonProject import plot_tz2


